/* Rank */
/* Returns the rank of each row within the partition of a result set.
 * The rank of a row is one plus the number of ranks that come before the row in question.
 */

USE AdventureWorks;
GO
SELECT i.ProductID, p.Name, i.LocationID, i.Quantity,
RANK() OVER (PARTITION BY i.LocationID order by i.Quantity) as RANK
FROM Production.ProductInventory i JOIN Production.Product p 
ON i.ProductID = p.ProductID
ORDER BY p.Name
GO

/* Dense Rank */
/* Returns the rank of rows within the partition of a result set,
 * without any gaps in the ranking. The rank of a row is one plus
 * the number of distinct ranks that come before the row in question.
 */

USE AdventureWorks;
GO
SELECT  i.ProductID, p.Name, i.LocationID, i.Quantity,
DENSE_RANK() OVER (PARTITION BY i.LocationID order by i.Quantity) as DENSE_RANK
FROM Production.ProductInventory i JOIN Production.Product p ON i.ProductID = p.ProductID
ORDER BY Name;
GO

/* NTile */
/* Distributes the rows in an ordered partition into a specified
 * number of groups. The groups are numbered, starting at one. For each
 * row, NTILE returns the number of the group to which the row belongs. 
 */

USE AdventureWorks;
GO
SELECT c.FirstName, c.LastName,
NTILE(4) OVER(ORDER BY SalesYTD DESC) AS 'Quartile',
s.SalesYTD, a.PostalCode
From Sales.SalesPerson s JOIN Person.Contact c on s.SalesPersonID = c.ContactID
JOIN Person.Address a ON a.AddressID = c.ContactID
WHERE TerritoryID IS NOT NULL AND SalesYTD <> 0;
GO

/* Row Number */
/* Returns the sequential number of a row within a partition of 
 * a result set, starting at 1 for the first row in each partition.
 */
 
USE AdventureWorks
GO
SELECT c.FirstName, c.LastName,
ROW_NUMBER() OVER(ORDER BY SalesYTD DESC) AS 'Row Number',
s.SalesYTD, a.PostalCode
FROM Sales.SalesPerson s JOIN Person.Contact c on s.SalesPersonID = c.ContactID
JOIN Person.Address a ON a.AddressID = c.ContactID
WHERE TerritoryID IS NOT NULL AND SalesYTD <> 0
GO
